@@ -47,13 +47,19 @@
 #include <uv.h>
 
 #ifdef XMRIG_FEATURE_HWLOC
-#include "base/kernel/Platform.h"
-#include "backend/cpu/platform/HwlocCpuInfo.h"
-#include <hwloc.h>
+#   include "base/kernel/Platform.h"
+#   include <hwloc.h>
+
+#   if HWLOC_API_VERSION < 0x20000
+#       define HWLOC_OBJ_L3CACHE HWLOC_OBJ_CACHE
+#   endif
 #endif
 
 #if defined(XMRIG_ARM)
 #   include "crypto/cn/sse2neon.h"
+#elif defined(XMRIG_RISCV)
+    // RISC-V doesn't have SSE/NEON, provide minimal compatibility
+#   define _mm_pause() __asm__ __volatile__("nop")
 #elif defined(__GNUC__)
 #   include <x86intrin.h>
 #else
@@ -86,7 +92,7 @@ CORE_HASH(14, whirlpool  );
 
 #undef CORE_HASH
 
-typedef void (*core_hash_func)(const uint8_t* data, size_t size, uint8_t* output);
+using core_hash_func = void (*)(const uint8_t* data, size_t size, uint8_t* output);
 static const core_hash_func core_hash[15] = { h0, h1, h2, h3, h4, h5, h6, h7, h8, h9, h10, h11, h12, h13, h14 };
 
 namespace xmrig
@@ -165,6 +171,8 @@ static struct AlgoTune
 
 struct HelperThread
 {
+    XMRIG_DISABLE_COPY_MOVE_DEFAULT(HelperThread)
+
     HelperThread(hwloc_bitmap_t cpu_set, int priority, bool is8MB) : m_cpuSet(cpu_set), m_priority(priority), m_is8MB(is8MB)
     {
         uv_mutex_init(&m_mutex);
@@ -194,14 +202,17 @@ struct HelperThread
 
     struct TaskBase
     {
-        virtual ~TaskBase() {}
-        virtual void run() = 0;
+        XMRIG_DISABLE_COPY_MOVE(TaskBase)
+
+        TaskBase()          = default;
+        virtual ~TaskBase() = default;
+        virtual void run()  = 0;
     };
 
     template<typename T>
     struct Task : TaskBase
     {
-        inline Task(T&& task) : m_task(std::move(task))
+        explicit inline Task(T&& task) : m_task(std::move(task))
         {
             static_assert(sizeof(Task) <= 128, "Task struct is too large");
         }
@@ -219,7 +230,7 @@ struct HelperThread
     inline void launch_task(T&& task)
     {
         uv_mutex_lock(&m_mutex);
-        new (&m_tasks[m_numTasks++]) Task<T>(std::move(task));
+        new (&m_tasks[m_numTasks++]) Task<T>(std::forward<T>(task));
         uv_cond_signal(&m_cond);
         uv_mutex_unlock(&m_mutex);
     }
@@ -234,7 +245,7 @@ struct HelperThread
     void run()
     {
         if (hwloc_bitmap_weight(m_cpuSet) > 0) {
-            hwloc_topology_t topology = reinterpret_cast<HwlocCpuInfo*>(Cpu::info())->topology();
+            hwloc_topology_t topology = Cpu::info()->topology();
             if (hwloc_set_cpubind(topology, m_cpuSet, HWLOC_CPUBIND_THREAD | HWLOC_CPUBIND_STRICT) < 0) {
                 hwloc_set_cpubind(topology, m_cpuSet, HWLOC_CPUBIND_THREAD);
             }
@@ -278,7 +289,7 @@ struct HelperThread
 
 void benchmark()
 {
-#ifndef XMRIG_ARM
+#if !defined(XMRIG_ARM) && !defined(XMRIG_RISCV)
     static std::atomic<int> done{ 0 };
     if (done.exchange(1)) {
         return;
@@ -288,9 +299,9 @@ void benchmark()
         // Try to avoid CPU core 0 because many system threads use it and can interfere
         uint32_t thread_index1 = (Cpu::info()->threads() > 2) ? 2 : 0;
 
-        hwloc_topology_t topology = reinterpret_cast<HwlocCpuInfo*>(Cpu::info())->topology();
+        hwloc_topology_t topology = Cpu::info()->topology();
         hwloc_obj_t pu = hwloc_get_pu_obj_by_os_index(topology, thread_index1);
-        hwloc_obj_t pu2;
+        hwloc_obj_t pu2 = nullptr;
         hwloc_get_closest_objs(topology, pu, &pu2, 1);
         uint32_t thread_index2 = pu2 ? pu2->os_index : thread_index1;
 
@@ -470,7 +481,7 @@ static inline bool findByType(hwloc_obj_t obj, hwloc_obj_type_t type, func lambd
 
 HelperThread* create_helper_thread(int64_t cpu_index, int priority, const std::vector<int64_t>& affinities)
 {
-#ifndef XMRIG_ARM
+#if !defined(XMRIG_ARM) && !defined(XMRIG_RISCV)
     hwloc_bitmap_t helper_cpu_set = hwloc_bitmap_alloc();
     hwloc_bitmap_t main_threads_set = hwloc_bitmap_alloc();
 
@@ -481,12 +492,17 @@ HelperThread* create_helper_thread(int64_t cpu_index, int priority, const std::v
     }
 
     if (cpu_index >= 0) {
-        hwloc_topology_t topology = reinterpret_cast<HwlocCpuInfo*>(Cpu::info())->topology();
-        hwloc_obj_t root = hwloc_get_root_obj(topology);
+        hwloc_obj_t root = hwloc_get_root_obj(Cpu::info()->topology());
 
         bool is8MB = false;
 
         findByType(root, HWLOC_OBJ_L3CACHE, [cpu_index, &is8MB](hwloc_obj_t obj) {
+#           if HWLOC_API_VERSION < 0x20000
+            if (obj->attr->cache.depth != 3) {
+                return false;
+            }
+#           endif
+
             if (!hwloc_bitmap_isset(obj->cpuset, cpu_index)) {
                 return false;
             }
@@ -510,7 +526,11 @@ HelperThread* create_helper_thread(int64_t cpu_index, int priority, const std::v
             return true;
         });
 
+#       if HWLOC_API_VERSION >= 0x20000
         for (auto obj_type : { HWLOC_OBJ_CORE, HWLOC_OBJ_L1CACHE, HWLOC_OBJ_L2CACHE, HWLOC_OBJ_L3CACHE }) {
+#       else
+        for (auto obj_type : { HWLOC_OBJ_CORE, HWLOC_OBJ_CACHE }) {
+#       endif
             findByType(root, obj_type, [cpu_index, helper_cpu_set, main_threads_set](hwloc_obj_t obj) {
                 const hwloc_cpuset_t& s = obj->cpuset;
                 if (hwloc_bitmap_isset(s, cpu_index)) {
@@ -571,9 +591,13 @@ void hash_octa(const uint8_t* data, size_t size, uint8_t* output, cryptonight_ct
     uint8_t tmp[64 * N];
 
     if (helper && (tune[cn_indices[0]].threads == 2) && (tune[cn_indices[1]].threads == 2) && (tune[cn_indices[2]].threads == 2)) {
-        const size_t n = N / 2;
+        constexpr size_t n = N / 2;
+
+        helper->launch_task([av, data, size, &ctx_memory, ctx, &cn_indices, &core_indices, &tmp, output, tune]() {
+#           ifdef _MSC_VER
+            constexpr size_t n = N / 2;
+#           endif
 
-        helper->launch_task([n, av, data, size, &ctx_memory, ctx, &cn_indices, &core_indices, &tmp, output, tune]() {
             const uint8_t* input = data;
             size_t input_size = size;
 
@@ -786,7 +810,7 @@ void hash_octa(const uint8_t* data, size_t size, uint8_t* output, cryptonight_ct
     uint32_t cn_indices[6];
     select_indices(cn_indices, seed);
 
-#ifdef XMRIG_ARM
+#if defined(XMRIG_ARM) || defined(XMRIG_RISCV)
     uint32_t step[6] = { 1, 1, 1, 1, 1, 1 };
 #else
     uint32_t step[6] = { 4, 4, 1, 2, 4, 4 };
